/*
* main.c
*
* Created: 30.03.2012 11:19:48
*  Author: maticpi
* 
* Description:
* Liquid level meter measures liquid level using a pressure sensor (simulated by a potentiometer connected to PA2).
* The result is displayed on the LCD screen and on a PC terminal via a RS232 connection. If the liquid level is too
* high or too low an alarm is engaged (LEDs 4 to 7 blink if level is too high and leds 0 to 3 blink if level is too low).
* Button T4 mutes the alarm until the condition, which triggered the alarm, disappears. The next time an alarm is 
* triggered it is not muted anymore.
*
* In a real system AD value 0 usually doesn't correspond to actual level 0 and maximum AD value doesn't correspond 
* to maximum liquid level. Therefore a calibration of the sensor is needed.
* Button T1 starts the calibration procedure. Next the sensor has to be set to 0 level (basically, pulled out of the 
* liquid), and then the T1 button has to be pressed again. The measured AD value corresponds to the actual liquid level 0.
* Next dip the sensor all the way to the bottom of the full liquid container and press the T1 button again. Now it records
* the AD value of the maximum liquid level.
* 
* The system can also receive commands from a PC via the RS232 connection. If neither alarm is active, character '0' 
* turns off all the LEDs, characters from '1' to '8' turn on LEDs from 0 to 7 (PB0 to PB7).
*
* Character 's' pauses normal liquid measuring operations and launches the settings menu. Here you can manually set the
* AD values for minimum and maximum liquid level, alarm thresholds and hysteresis and you can save all the data into EEPROM.
* These values are reloaded during initialization after reset.
* The menus are traversed using characters '1', '2', '3', 's' , ENTER(dec 13), 'x' and ESC(dec 27).
*
*
* 
*/ 

#include "main.h"

volatile long mstick=0;				//milliseconds since power-up
volatile unsigned char MyFLAGS=0;	//My flags (whatever takes my fancy...)

int LowAlarmLevel=30;				//Low alarm level threshold
int HighAlarmLevel=90;				//High alarm level threshold
int Hysteresis=5;					//Alarm hysteresis (the same for both)
int bEmpty=0;						//AD value for no liquid
int bFull=1023;						//AD value for maximum liquid level
int bSpan=1023;						//bFull-bEmpty

void checkForAlarm(int level);
void BlinkHigh();
void BlinkLow();

//glavna funkcija - main
int main(void)
{
	int ADvalue,i,oldADvalue=-1,percent,old_percent=-1;
	int cnt100ms=0;
	int graph16, old_graph16=100;

	Init();
	LCD_Init();
	stdout = &LCDstream;
	LCD_Control(LCD_ON | LCD_CURS_OFF);		//Turn off cursor on the LCD
	ReadFromEEPROM();						//Check EEPROM for stored settings and reload them

	while(1)
    {
		while (!MF10ms)
		{
			// do as fast as possible:
			ReadRS232();	//Read RS232, and react to commands if there are any. If not, just return.
		}
		Clr10msFlag
		// do once every 10 ms:             
		cnt100ms++;
		if (cnt100ms == 10)	
		{
			cnt100ms=0;
			//do once every 100 ms:
			ADvalue=AD_convert();						//Get liquid level
			if (ADvalue != oldADvalue)					//if different than before,
			{
				LCD_GotoXY(0,0);
				printf_P(PSTR("R%4d"),ADvalue);			//   refresh AD value on LCD
				RefreshRS232(ADvalue);					//   and RS232 (refreshes everything)
				oldADvalue=ADvalue;
			}
			ADvalue-=bEmpty;							//AD value can theoretically be more or less min and max values (noise, electrical error, naughty user,...)
			if (ADvalue < 0) ADvalue=0; 				//Adjust AD value to min and max values (bEmpty and bFull)
			if (ADvalue > bSpan) ADvalue=bSpan;
			percent=((ADvalue+bSpan/200)*100L)/bSpan;	//calculate percentage according to min and max values. 
														//Round the result by adding 0.5% (...+bSpan/(2*100)) to raw AD value.
			if (percent != old_percent)					//If percent changed
			{
				LCD_GotoXY(0,6);
				printf_P(PSTR("N%3d%%"),percent);		//   refresh it
				old_percent=percent;
			}

			graph16=(ADvalue+bSpan/32)*16/bSpan;
			if (graph16 != old_graph16)					//If graph length should change, refresh it
			{
				LCD_GotoXY(1,0);
				for (i=0; i<graph16; i++) LCD_Data(255);//Draw as many black boxes as needed,
				for (; i<16; i++) LCD_Data(' ');		//then continue with spaces (to overwrite any black boxes that may be there from before)
				old_graph16=graph16;			
				LCD_GotoXY(0,0);						//LCD_data() doesn't update x and y variables for LCD. Call LCD_GotoXY to update them (see implementation in lcd.c)
			}
			checkForAlarm(percent);						//Check if alarm has to be engaged and blink LEDs if so.
		}
		//Read and react to keyboard input
        readKYB();	
		switch (getKEY())
		{
			case 1:					//If T1 pressed, start calibration procedure
				Calibration();
				break;
			case 4:					//If T4 pressed, mute alarm
				
				if (MFAlarmHigh || MFAlarmLow)	//If any alarm active
				{
					SetMuteAlarm				//   mute alarm
					PORTB=0;					//   and clear PORTB
				}								//   otherwise ignore the button
				break;
			default:				//Ignore all other keys
				break;
		}                   
    }
}

//function for checking alarm conditions, and engaging alarms as necessary (also calls functions for blinking with LEDs)
void checkForAlarm(int level)
{
	//LowAlarm
	if (level<=LowAlarmLevel) SetAlarmLow;					//If level to low engage Low alarm.
	if (level>(LowAlarmLevel+Hysteresis) && MFAlarmLow)     //If low alarm is active AND the level has risen enough disengage alarm and clear PORTB
	{
		ClrAlarmLow;	//Only clear LEDs if an Alarm has been active until now. 
		PORTB&=~0x0F;	// Otherwise continuous clearing of the LEDs could interfere
	}					// with other functions which may also be using the LEDs.

	//HighAlarm
	if (level>=HighAlarmLevel) SetAlarmHigh;
	if (level<(HighAlarmLevel-Hysteresis) && MFAlarmHigh)
	{
		ClrAlarmHigh;
		PORTB&=~0xF0;
	}		
	
	//Clear Mute alarm if conditions for alarms are gone
	if ( (level<(HighAlarmLevel-Hysteresis)) && (level>(LowAlarmLevel+Hysteresis)) && MFmuteAlarm) ClrMuteAlarm;
	
	//If any alarms are active (and not muted), blink with LEDs
	if (MFAlarmLow && !MFmuteAlarm) BlinkLow(); 
	if (MFAlarmHigh && !MFmuteAlarm) BlinkHigh();
}

//function for blinking with LEDs (for high alarm)
//No delays must be used because that would interfere with operation of other functions
//Function is run every 100ms - this fact is used for time measurement
void BlinkHigh()
{
	static int delay=0;
	delay++;
	if ( (PORTB&0xF0) && (delay >= 3) )	//If LEDs are on, wait 300ms
	{
		PORTB&=~0xF0;
		delay=0;
	}			
	else if (delay >= 3) //else, also wait 300ms
	{
		PORTB|=0xF0;
		delay=0;
	}			
}

//function for blinking with LEDs (for low alarm)
//No delays must be used because that would interfere with operation of other functions
//Function is run every 100ms - this fact is used for time measurement
void BlinkLow()
{
	static int delay=0;
	delay++;
	if (PORTB&0x0F) 
	{
		if (delay >= 4)		//If LEDs are on, wait 400ms
		{
			PORTB&=~0x0F;
			delay=0;
		}			
	}	
	else 
	{
		if (delay >= 2) //else, wait 200ms
		{
			PORTB|=0x0F;
			delay=0;
		}			
	}		
}
